@@ -34,6 +34,7 @@ pub use self::{
     rows_event::{RowsEvent, RowsEventRows},
     rows_query_event::RowsQueryEvent,
     table_map_event::*,
+    transaction_payload_event::{TransactionPayloadEvent, TransactionPayloadReader},
     update_rows_event::UpdateRowsEvent,
     update_rows_event_v1::UpdateRowsEventV1,
     user_var_event::UserVarEvent,
@@ -52,8 +53,8 @@ use std::{
 
 use super::{
     consts::{
-        BinlogChecksumAlg, BinlogVersion, EventFlags, EventType, UnknownChecksumAlg,
-        UnknownEventType,
+        BinlogChecksumAlg, BinlogVersion, EventFlags, EventType, RowsEventFlags,
+        UnknownChecksumAlg, UnknownEventType,
     },
     misc::LimitWrite,
     BinlogCtx, BinlogEvent,
@@ -75,6 +76,7 @@ mod rotate_event;
 mod rows_event;
 mod rows_query_event;
 mod table_map_event;
+mod transaction_payload_event;
 mod update_rows_event;
 mod update_rows_event_v1;
 mod user_var_event;
@@ -140,9 +142,11 @@ impl Event {
             fde.footer()
         };
 
-        // fde will always contain checksum (see WL#2540)
+        // * fde will always contain checksum (see WL#2540)
+        // * events inside of a Transaction_payload_event are not checksummed (see WL#3549)
         let contains_checksum = footer.checksum_alg.is_some()
-            && (is_fde || footer.checksum_alg != Some(RawConst::new(0)));
+            && (is_fde || footer.checksum_alg != Some(RawConst::new(0)))
+            && footer.checksum_enabled;
 
         if contains_checksum {
             // truncate checksum
@@ -183,7 +187,7 @@ impl Event {
         Ok(())
     }
 
-    /// Returns a length of a serialized represenation of this event.
+    /// Returns a length of a serialized representation of this event.
     fn len(&self, _version: BinlogVersion) -> usize {
         let is_fde = self.header.event_type.0 == EventType::FORMAT_DESCRIPTION_EVENT as u8;
         let mut len = S(0);
@@ -227,7 +231,7 @@ impl Event {
         let contains_checksum = self.footer.checksum_alg.is_some()
             && (self.header.event_type.0 == (EventType::FORMAT_DESCRIPTION_EVENT as u8)
                 || self.footer.checksum_alg != Some(RawConst::new(0)));
-        contains_checksum.then(|| self.checksum)
+        contains_checksum.then_some(self.checksum)
     }
 
     /// Read event-type specific data as a binlog struct.
@@ -324,6 +328,7 @@ impl Event {
             PARTIAL_UPDATE_ROWS_EVENT => {
                 EventData::RowsEvent(RowsEventData::PartialUpdateRowsEvent(self.read_event()?))
             }
+            TRANSACTION_PAYLOAD_EVENT => EventData::TransactionPayloadEvent(self.read_event()?),
         };
 
         Ok(Some(event_data))
@@ -435,15 +440,15 @@ impl BinlogEventHeader {
         self.log_pos.0
     }
 
-    /// Returns the raw event flags.
-    pub fn flags_raw(&self) -> u16 {
-        self.flags.0
-    }
-
-    /// Returns the parsed event flags. Unkonwn bits will be truncated.
+    /// Returns event flags (unknown bits are truncated).
     pub fn flags(&self) -> EventFlags {
         self.flags.get()
     }
+
+    /// Returns raw event flags (unknown bits are preserved).
+    pub fn flags_raw(&self) -> u16 {
+        self.flags.0
+    }
 }
 
 impl<'de> MyDeserialize<'de> for BinlogEventHeader {
@@ -479,6 +484,9 @@ impl MySerialize for BinlogEventHeader {
 pub struct BinlogEventFooter {
     /// Raw checksum algorithm description.
     checksum_alg: Option<RawConst<u8, BinlogChecksumAlg>>,
+
+    /// Checksum enabled
+    checksum_enabled: bool,
 }
 
 impl BinlogEventFooter {
@@ -492,6 +500,7 @@ impl BinlogEventFooter {
     pub fn new(checksum_alg: BinlogChecksumAlg) -> Self {
         Self {
             checksum_alg: Some(RawConst::new(checksum_alg as u8)),
+            checksum_enabled: true,
         }
     }
 
@@ -500,6 +509,16 @@ impl BinlogEventFooter {
         self.checksum_alg.as_ref().map(RawConst::get).transpose()
     }
 
+    /// Returns `true` if checksum is enabled.
+    pub fn get_checksum_enabled(&self) -> bool {
+        self.checksum_enabled
+    }
+
+    /// Set checksum enabled flag.
+    pub fn set_checksum_enabled(&mut self, enabled: bool) {
+        self.checksum_enabled = enabled;
+    }
+
     /// Reads binlog event footer from the given buffer.
     ///
     /// Requires that buf contains `FormatDescriptionEvent` data.
@@ -525,6 +544,7 @@ impl BinlogEventFooter {
 
         Ok(Self {
             checksum_alg: checksum_alg.map(RawConst::new),
+            checksum_enabled: true,
         })
     }
 }
@@ -535,6 +555,7 @@ impl Default for BinlogEventFooter {
             checksum_alg: Some(RawConst::new(
                 BinlogChecksumAlg::BINLOG_CHECKSUM_ALG_OFF as u8,
             )),
+            checksum_enabled: true,
         }
     }
 }
@@ -578,7 +599,6 @@ pub enum EventData<'a> {
     HeartbeatEvent,
     IgnorableEvent(Cow<'a, [u8]>),
     RowsQueryEvent(RowsQueryEvent<'a>),
-    /// Not yet implemented.
     GtidEvent(GtidEvent),
     /// Not yet implemented.
     AnonymousGtidEvent(AnonymousGtidEvent),
@@ -591,6 +611,7 @@ pub enum EventData<'a> {
     /// Not yet implemented.
     XaPrepareLogEvent(Cow<'a, [u8]>),
     RowsEvent(RowsEventData<'a>),
+    TransactionPayloadEvent(TransactionPayloadEvent<'a>),
 }
 
 impl<'a> EventData<'a> {
@@ -642,6 +663,9 @@ impl<'a> EventData<'a> {
                 EventData::XaPrepareLogEvent(Cow::Owned(ev.into_owned()))
             }
             Self::RowsEvent(ev) => EventData::RowsEvent(ev.into_owned()),
+            Self::TransactionPayloadEvent(ev) => {
+                EventData::TransactionPayloadEvent(ev.into_owned())
+            }
         }
     }
 }
@@ -650,18 +674,18 @@ impl MySerialize for EventData<'_> {
     fn serialize(&self, buf: &mut Vec<u8>) {
         match self {
             EventData::UnknownEvent => (),
-            EventData::StartEventV3(ev) => buf.put_slice(&*ev),
+            EventData::StartEventV3(ev) => buf.put_slice(ev),
             EventData::QueryEvent(ev) => ev.serialize(buf),
             EventData::StopEvent => (),
             EventData::RotateEvent(ev) => ev.serialize(buf),
             EventData::IntvarEvent(ev) => ev.serialize(buf),
-            EventData::LoadEvent(ev) => buf.put_slice(&*ev),
+            EventData::LoadEvent(ev) => buf.put_slice(ev),
             EventData::SlaveEvent => (),
-            EventData::CreateFileEvent(ev) => buf.put_slice(&*ev),
-            EventData::AppendBlockEvent(ev) => buf.put_slice(&*ev),
-            EventData::ExecLoadEvent(ev) => buf.put_slice(&*ev),
-            EventData::DeleteFileEvent(ev) => buf.put_slice(&*ev),
-            EventData::NewLoadEvent(ev) => buf.put_slice(&*ev),
+            EventData::CreateFileEvent(ev) => buf.put_slice(ev),
+            EventData::AppendBlockEvent(ev) => buf.put_slice(ev),
+            EventData::ExecLoadEvent(ev) => buf.put_slice(ev),
+            EventData::DeleteFileEvent(ev) => buf.put_slice(ev),
+            EventData::NewLoadEvent(ev) => buf.put_slice(ev),
             EventData::RandEvent(ev) => ev.serialize(buf),
             EventData::UserVarEvent(ev) => ev.serialize(buf),
             EventData::FormatDescriptionEvent(ev) => ev.serialize(buf),
@@ -669,20 +693,21 @@ impl MySerialize for EventData<'_> {
             EventData::BeginLoadQueryEvent(ev) => ev.serialize(buf),
             EventData::ExecuteLoadQueryEvent(ev) => ev.serialize(buf),
             EventData::TableMapEvent(ev) => ev.serialize(buf),
-            EventData::PreGaWriteRowsEvent(ev) => buf.put_slice(&*ev),
-            EventData::PreGaUpdateRowsEvent(ev) => buf.put_slice(&*ev),
-            EventData::PreGaDeleteRowsEvent(ev) => buf.put_slice(&*ev),
+            EventData::PreGaWriteRowsEvent(ev) => buf.put_slice(ev),
+            EventData::PreGaUpdateRowsEvent(ev) => buf.put_slice(ev),
+            EventData::PreGaDeleteRowsEvent(ev) => buf.put_slice(ev),
             EventData::IncidentEvent(ev) => ev.serialize(buf),
             EventData::HeartbeatEvent => (),
-            EventData::IgnorableEvent(ev) => buf.put_slice(&*ev),
+            EventData::IgnorableEvent(ev) => buf.put_slice(ev),
             EventData::RowsQueryEvent(ev) => ev.serialize(buf),
             EventData::GtidEvent(ev) => ev.serialize(buf),
             EventData::AnonymousGtidEvent(ev) => ev.serialize(buf),
-            EventData::PreviousGtidsEvent(ev) => buf.put_slice(&*ev),
-            EventData::TransactionContextEvent(ev) => buf.put_slice(&*ev),
-            EventData::ViewChangeEvent(ev) => buf.put_slice(&*ev),
-            EventData::XaPrepareLogEvent(ev) => buf.put_slice(&*ev),
+            EventData::PreviousGtidsEvent(ev) => buf.put_slice(ev),
+            EventData::TransactionContextEvent(ev) => buf.put_slice(ev),
+            EventData::ViewChangeEvent(ev) => buf.put_slice(ev),
+            EventData::XaPrepareLogEvent(ev) => buf.put_slice(ev),
             EventData::RowsEvent(ev) => ev.serialize(buf),
+            EventData::TransactionPayloadEvent(ev) => ev.serialize(buf),
         }
     }
 }
@@ -729,7 +754,7 @@ impl<'a> RowsEventData<'a> {
     /// Returns columns in the before-image (only for DELETE and UPDATE).
     ///
     /// Each bit indicates whether corresponding column is used in the image.
-    pub fn columns_before_image(&'a self) -> Option<&'a BitSlice<Lsb0, u8>> {
+    pub fn columns_before_image(&'a self) -> Option<&'a BitSlice<u8>> {
         match self {
             RowsEventData::WriteRowsEventV1(_) => None,
             RowsEventData::UpdateRowsEventV1(ev) => Some(ev.columns_before_image()),
@@ -744,7 +769,7 @@ impl<'a> RowsEventData<'a> {
     /// Returns columns in the after-image (only for WRITE and UPDATE).
     ///
     /// Each bit indicates whether corresponding column is used in the image.
-    pub fn columns_after_image(&'a self) -> Option<&'a BitSlice<Lsb0, u8>> {
+    pub fn columns_after_image(&'a self) -> Option<&'a BitSlice<u8>> {
         match self {
             RowsEventData::WriteRowsEventV1(ev) => Some(ev.columns_after_image()),
             RowsEventData::UpdateRowsEventV1(ev) => Some(ev.columns_after_image()),
@@ -769,6 +794,19 @@ impl<'a> RowsEventData<'a> {
         }
     }
 
+    /// Returns event flags.
+    pub fn flags(&self) -> RowsEventFlags {
+        match self {
+            RowsEventData::WriteRowsEventV1(ev) => ev.flags(),
+            RowsEventData::UpdateRowsEventV1(ev) => ev.flags(),
+            RowsEventData::DeleteRowsEventV1(ev) => ev.flags(),
+            RowsEventData::WriteRowsEvent(ev) => ev.flags(),
+            RowsEventData::UpdateRowsEvent(ev) => ev.flags(),
+            RowsEventData::DeleteRowsEvent(ev) => ev.flags(),
+            RowsEventData::PartialUpdateRowsEvent(ev) => ev.flags(),
+        }
+    }
+
     /// Returns an iterator over event's rows given the corresponding `TableMapEvent`.
     pub fn rows(&'a self, table_map_event: &'a TableMapEvent<'a>) -> RowsEventRows<'a> {
         match self {
