@@ -19,7 +19,7 @@ use std::{
     convert::TryFrom,
     hash::Hash,
     io::{
-        self, Error,
+        self, BufRead, Error,
         ErrorKind::{InvalidData, UnexpectedEof},
         Read, Write,
     },
@@ -30,9 +30,12 @@ use crate::{
     proto::{MyDeserialize, MySerialize},
 };
 
+#[allow(unused)]
+use self::events::TransactionPayloadEvent;
+
 use self::{
     consts::{BinlogVersion, EventType},
-    events::{Event, FormatDescriptionEvent, TableMapEvent},
+    events::{Event, FormatDescriptionEvent, RotateEvent, TableMapEvent},
 };
 
 pub mod consts;
@@ -100,9 +103,11 @@ impl BinlogFileHeader {
 
 /// Reader for binlog events.
 ///
-/// It'll maintain actual fde and table map, and can be used
-/// to read binlog files and binlog event streams from server.
-#[derive(Debug, Clone, Eq, PartialEq)]
+/// # Note
+///
+/// It's a low-level stream reader and only maintains actual fde and table map,
+/// so one should properly handle encountered events (see docs on [`EventStreamReader::read`]).
+#[derive(Debug)]
 pub struct EventStreamReader {
     fde: FormatDescriptionEvent<'static>,
     table_map: HashMap<u64, TableMapEvent<'static>>,
@@ -124,6 +129,13 @@ impl EventStreamReader {
         &self.fde
     }
 
+    /// Disable/Enable checksum verification without changing the original algorithm.
+    ///
+    /// See [`EventStreamReader::read_decompressed`].
+    pub(crate) fn set_checksum_enabled(&mut self, enabled: bool) {
+        self.fde.footer_mut().set_checksum_enabled(enabled);
+    }
+
     /// Returns the table map event for the given table id.
     ///
     /// Should be availeble if rows event with this table id encountered in the stream.
@@ -131,41 +143,99 @@ impl EventStreamReader {
         self.table_map.get(&table_id)
     }
 
-    /// Will read next event from the given stream.
-    pub fn read<T: Read>(&mut self, input: T) -> io::Result<Event> {
+    /// Will read next event from the given stream (Returns None if stream is exhausted).
+    ///
+    /// # Note
+    ///
+    /// Since MySql 8.0.20 it is possible for an event stream to contain an embedded
+    /// stream of events in form of a [`TransactionPayloadEvent`]. This means that
+    /// to properly handle table maps it is necessary to read the embedded stream
+    /// as soon as it is encountered (see [`EventStreamReader::read_decompressed`]).
+    pub fn read<T: BufRead>(&mut self, mut input: T) -> io::Result<Option<Event>> {
+        if input.fill_buf().map(|x| x.is_empty())? {
+            return Ok(None);
+        }
+
         let event = Event::read(&self.fde, input)?;
+
+        self.handle_event(&event)?;
+
+        Ok(Some(event))
+    }
+
+    /// This function reads decompressed payload of a Transaction_payload_event
+    /// (see [`TransactionPayloadEvent::decompressed`]).
+    ///
+    /// The difference is that checksum verification will be disabled according to the WL#3549.
+    ///
+    /// # Warning
+    ///
+    /// This function can't be used to skip checksum verification for regular events.
+    ///
+    /// # Errors
+    ///
+    /// There is a list of events that should never be a part of a transaction payload and the list
+    /// includes the [`TransactionPayloadEvent`] itself.
+    /// This function will emit an [`io::ErrorKind::Other`] if [`TransactionPayloadEvent`]
+    /// is encountered within the compressed payload.
+    pub fn read_decompressed<T: BufRead>(&mut self, input: T) -> io::Result<Option<Event>> {
+        self.set_checksum_enabled(false);
+        let result = self.read(input);
+        self.set_checksum_enabled(true);
+        let Some(event) = result? else {
+            return Ok(None);
+        };
+
+        if event.header().event_type_raw() == EventType::TRANSACTION_PAYLOAD_EVENT as u8 {
+            return Err(io::Error::new(
+                io::ErrorKind::Other,
+                "TRANSACTION_PAYLOAD_EVENT encountered",
+            ));
+        }
+
+        self.handle_event(&event)?;
+
+        Ok(Some(event))
+    }
+
+    fn handle_event(&mut self, event: &Event) -> io::Result<()> {
         let event_type = event.header().event_type_raw();
 
         if event_type == EventType::FORMAT_DESCRIPTION_EVENT as u8 {
             // we'll redefine fde with an actual one
-            self.fde = match event.read_event::<FormatDescriptionEvent>() {
-                Ok(fde) => fde.into_owned().with_footer(event.footer()),
-                Err(err) => return Err(err),
-            };
+            let fde = event.read_event::<FormatDescriptionEvent>()?;
+            self.fde = fde.into_owned().with_footer(event.footer());
         } else if event_type == EventType::TABLE_MAP_EVENT as u8 {
             // we'll maintain known table maps
-            match event.read_event::<TableMapEvent>() {
-                Ok(tme) => {
-                    self.table_map.insert(tme.table_id(), tme.into_owned());
-                }
-                Err(err) => return Err(err),
+            let tme = event.read_event::<TableMapEvent>()?;
+            self.table_map.insert(tme.table_id(), tme.into_owned());
+        } else if event_type == EventType::ROTATE_EVENT as u8 {
+            // we'll keep table map size within reasonlable bounds
+
+            // TODO: This value is arbitrary
+            const TABLE_MAP_MAX_SIZE: usize = 64;
+
+            let re = event.read_event::<RotateEvent>()?;
+            if !re.is_fake() {
+                self.table_map.clear();
+                self.table_map.shrink_to(TABLE_MAP_MAX_SIZE);
             }
         }
 
-        Ok(event)
+        Ok(())
     }
 }
 
 /// Binlog file.
 ///
 /// It's an iterator over events in a binlog file.
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug)]
 pub struct BinlogFile<T> {
     reader: EventStreamReader,
     read: T,
 }
 
-impl<T: Read> BinlogFile<T> {
+impl<T: BufRead> BinlogFile<T> {
     /// Creates a new instance.
     ///
     /// It'll try to read binlog file header.
@@ -179,14 +249,19 @@ impl<T: Read> BinlogFile<T> {
     pub fn reader(&self) -> &EventStreamReader {
         &self.reader
     }
+
+    /// Returns a mutable reference to the binlog stream reader.
+    pub fn reader_mut(&mut self) -> &mut EventStreamReader {
+        &mut self.reader
+    }
 }
 
-impl<T: Read> Iterator for BinlogFile<T> {
+impl<T: BufRead> Iterator for BinlogFile<T> {
     type Item = io::Result<Event>;
 
     fn next(&mut self) -> Option<Self::Item> {
         match self.reader.read(&mut self.read) {
-            Ok(event) => Some(Ok(event)),
+            Ok(event) => event.map(Ok),
             Err(err) if err.kind() == UnexpectedEof => None,
             Err(err) => Some(Err(err)),
         }
@@ -223,9 +298,9 @@ impl ColumnType {
             | Self::MYSQL_TYPE_ENUM
             | Self::MYSQL_TYPE_STRING
             | Self::MYSQL_TYPE_BIT => ptr.get(..2).map(|x| (x, 2)),
-            Self::MYSQL_TYPE_TYPED_ARRAY => Self::try_from(*ptr.get(0)?)
+            Self::MYSQL_TYPE_TYPED_ARRAY => Self::try_from(*ptr.first()?)
                 .ok()?
-                .get_metadata(&ptr.get(1..)?, true)
+                .get_metadata(ptr.get(1..)?, true)
                 .map(|(x, n)| (x, n + 1)),
             _ => Some((&[], 0)),
         }
@@ -234,7 +309,11 @@ impl ColumnType {
 
 #[cfg(test)]
 mod tests {
-    use std::{collections::HashMap, io};
+    use std::{
+        collections::HashMap,
+        io,
+        iter::{once, repeat},
+    };
 
     use super::{
         consts::{EventFlags, EventType},
@@ -242,7 +321,13 @@ mod tests {
         BinlogFile, BinlogFileHeader, BinlogVersion,
     };
 
-    use crate::proto::MySerialize;
+    use crate::{
+        binlog::{events::RowsEventData, value::BinlogValue},
+        collations::CollationId,
+        constants::ColumnFlags,
+        proto::MySerialize,
+        value::Value,
+    };
 
     const BINLOG_FILE: &[u8] = &[
         0xfe, 0x62, 0x69, 0x6e, 0xfc, 0x35, 0xbb, 0x4a, 0x0f, 0x01, 0x00, 0x00, 0x00, 0x5e, 0x00,
@@ -639,6 +724,7 @@ mod tests {
 
             while let Some(ev) = binlog_file.next() {
                 let ev = ev?;
+                let _ = dbg!(ev.header().event_type());
                 let ev_end = ev_pos + ev.header().event_size() as usize;
                 let binlog_version = binlog_file.reader.fde.binlog_version();
 
@@ -647,10 +733,17 @@ mod tests {
 
                 let event = match ev.read_data() {
                     Ok(event) => {
-                        let event = event.unwrap_or_else(|| {
-                            dbg!(&ev);
-                            panic!()
-                        });
+                        let event = match event {
+                            Some(e) => e,
+                            None => {
+                                if file_path.file_name().unwrap() == "mariadb-bin.000001" {
+                                    continue;
+                                } else {
+                                    dbg!(&ev);
+                                    panic!();
+                                }
+                            }
+                        };
                         match event {
                             EventData::TableMapEvent(ref ev) => {
                                 // store table maps for later use
@@ -663,7 +756,45 @@ mod tests {
                                 let table_map_event =
                                     binlog_file.reader().get_tme(rows_event.table_id()).unwrap();
                                 for row in rows_event.rows(table_map_event) {
-                                    row.unwrap();
+                                    let _row = row.unwrap();
+                                    if file_path.file_name().unwrap() == "mariadb-bin.000001" {
+                                        // should parse metadata for `binlog_row_metadata=FULL`
+                                        let after = _row.1.as_ref().unwrap();
+                                        let columns = after.columns_ref();
+
+                                        for col in columns.iter() {
+                                            assert_eq!(col.schema_ref(), b"toddy_test");
+                                            assert_eq!(col.table_ref(), b"outbox");
+                                            assert_eq!(col.org_table_ref(), b"outbox");
+                                        }
+
+                                        for (col, col_name) in columns.iter().zip([
+                                            "id",
+                                            "topic",
+                                            "event_type",
+                                            "event",
+                                            "created",
+                                        ]) {
+                                            assert_eq!(col.name_ref(), col_name.as_bytes());
+                                        }
+
+                                        for (col, f) in columns.iter().zip(
+                                            once(ColumnFlags::PRI_KEY_FLAG)
+                                                .chain(repeat(ColumnFlags::empty())),
+                                        ) {
+                                            assert_eq!(col.flags(), f);
+                                        }
+
+                                        for (col, charset) in columns.iter().zip([
+                                            CollationId::UNKNOWN_COLLATION_ID,
+                                            CollationId::UTF8MB4_GENERAL_CI,
+                                            CollationId::UTF8MB4_GENERAL_CI,
+                                            CollationId::BINARY,
+                                            CollationId::UNKNOWN_COLLATION_ID,
+                                        ]) {
+                                            assert_eq!(col.character_set(), charset as u16);
+                                        }
+                                    }
                                 }
 
                                 event
@@ -694,16 +825,150 @@ mod tests {
 
                 if file_path.file_name().unwrap() == "binlog-invisible-columns.000001" {
                     if let Some(EventData::TableMapEvent(ev)) = ev.read_data().unwrap() {
-                        dbg!(&ev);
-                        let optional_meta = dbg!(ev.iter_optional_meta());
+                        let optional_meta = ev.iter_optional_meta();
                         for meta in optional_meta {
-                            dbg!(meta.unwrap());
+                            meta.unwrap();
                         }
                     }
                 }
 
-                assert_eq!(output, &file_data[ev_pos..ev_end]);
+                if file_path.file_name().unwrap() == "mysql-enum-string-set.000001" {
+                    if let Some(EventData::RowsEvent(data)) = ev.read_data().unwrap() {
+                        let table_map_event =
+                            binlog_file.reader().get_tme(data.table_id()).unwrap();
+                        for row in data.rows(table_map_event) {
+                            let (before, after) = row.unwrap();
+                            match data {
+                                RowsEventData::WriteRowsEvent(_) => {
+                                    assert!(before.is_none());
+                                    let after = after.unwrap().unwrap();
+                                    let mut j = 0;
+                                    for v in after {
+                                        j += 1;
+                                        match j {
+                                            1 => assert_eq!(v, BinlogValue::Value("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789".into())),
+                                            2 => assert_eq!(v, BinlogValue::Value("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789".into())),
+                                            3 => assert_eq!(v, BinlogValue::Value(1_i8.into())),
+                                            4 => assert_eq!(v, BinlogValue::Value([0b00000101_u8].into())),
+                                            5 => assert_eq!(v, BinlogValue::Value("0123456789".into())),
 
+                                            _ => panic!(),
+                                        }
+                                    }
+                                    assert_eq!(j, 5);
+                                }
+                                RowsEventData::UpdateRowsEvent(_) => {
+                                    let before = before.unwrap().unwrap();
+                                    let mut j = 0;
+                                    for v in before {
+                                        j += 1;
+                                        match j {
+                                            1 => assert_eq!(v, BinlogValue::Value("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789".into())),
+                                            2 => assert_eq!(v, BinlogValue::Value("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789".into())),
+                                            3 => assert_eq!(v, BinlogValue::Value(1_i8.into())),
+                                            4 => assert_eq!(v, BinlogValue::Value([0b00000101_u8].into())),
+                                            5 => assert_eq!(v, BinlogValue::Value("0123456789".into())),
+
+                                            _ => panic!(),
+                                        }
+                                    }
+                                    assert_eq!(j, 5);
+
+                                    let after = after.unwrap().unwrap();
+                                    let mut j = 0;
+                                    for v in after {
+                                        j += 1;
+                                        match j {
+                                            1 => assert_eq!(v, BinlogValue::Value("field1".into())),
+                                            2 => assert_eq!(v, BinlogValue::Value("field_2".into())),
+                                            3 => assert_eq!(v, BinlogValue::Value(2_i8.into())),
+                                            4 => assert_eq!(v, BinlogValue::Value([0b00001010_u8].into())),
+                                            5 => assert_eq!(v, BinlogValue::Value("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789".into())),
+                                            _ => panic!(),
+                                        }
+                                    }
+                                    assert_eq!(j, 5);
+                                }
+                                RowsEventData::DeleteRowsEvent(_) => {
+                                    assert!(after.is_none());
+
+                                    let before = before.unwrap().unwrap();
+                                    let mut j = 0;
+                                    for v in before {
+                                        j += 1;
+                                        match j {
+                                            1 => assert_eq!(v, BinlogValue::Value("field1".into())),
+                                            2 => assert_eq!(v, BinlogValue::Value("field_2".into())),
+                                            3 => assert_eq!(v, BinlogValue::Value(2_i8.into())),
+                                            4 => assert_eq!(v, BinlogValue::Value([0b00001010_u8].into())),
+                                            5 => assert_eq!(v, BinlogValue::Value("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456780123456789012345678901234567890123456789".into())),
+                                            _ => panic!(),
+                                        }
+                                    }
+                                    assert_eq!(j, 5);
+                                }
+                                _ => panic!(),
+                            }
+                        }
+                    }
+                }
+
+                if file_path.file_name().unwrap() == "mariadb-bin.000001" {
+                    // Extraneous bytes in RotateEvent file name
+                    // https://github.com/blackbeam/mysql_async/issues/189
+                    if let Some(EventData::RotateEvent(ev)) = ev.read_data().unwrap() {
+                        assert_ne!(ev.name_raw(), b"mariadb-bin.000001");
+                    }
+                }
+
+                if file_path.file_name().unwrap() == "mysql_type_bit.000001" {
+                    if let Some(EventData::RowsEvent(ev)) = ev.read_data().unwrap() {
+                        let table_map_event = binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                        for row in ev.rows(table_map_event) {
+                            let (before, after) = row.unwrap();
+                            assert_eq!(before, None);
+                            assert_eq!(
+                                after.unwrap().unwrap(),
+                                vec![
+                                    BinlogValue::Value(Value::Bytes(vec![0b100])),
+                                    BinlogValue::Value(Value::Bytes(b"foo".to_vec())),
+                                    BinlogValue::Value(Value::Bytes(vec![0b100000])),
+                                ],
+                            );
+                        }
+                    }
+                }
+
+                if file_path.file_name().unwrap() != "mariadb-bin.000001" {
+                    assert_eq!(output, &file_data[ev_pos..ev_end]);
+                }
+
+                if file_path.file_name().unwrap() == "transaction_compression.000001" {
+                    if let Some(EventData::TransactionPayloadEvent(data)) = ev.read_data().unwrap()
+                    {
+                        let mut payload = data.decompressed()?;
+                        let reader = binlog_file.reader_mut();
+
+                        let mut binlog_ev = reader.read_decompressed(&mut payload)?.unwrap();
+                        assert_eq!(binlog_ev.header().event_type(), Ok(EventType::QUERY_EVENT));
+
+                        binlog_ev = reader.read_decompressed(&mut payload)?.unwrap();
+                        assert_eq!(
+                            binlog_ev.header().event_type(),
+                            Ok(EventType::TABLE_MAP_EVENT)
+                        );
+
+                        binlog_ev = reader.read_decompressed(&mut payload)?.unwrap();
+                        assert_eq!(
+                            binlog_ev.header().event_type(),
+                            Ok(EventType::WRITE_ROWS_EVENT)
+                        );
+
+                        binlog_ev = reader.read_decompressed(&mut payload)?.unwrap();
+                        assert_eq!(binlog_ev.header().event_type(), Ok(EventType::XID_EVENT));
+                        assert!(reader.read_decompressed(&mut payload)?.is_none());
+                    }
+                }
                 output = Vec::new();
                 event.serialize(&mut output);
 
