@@ -17,6 +17,8 @@ use crate::{
     value::Value::{self, *},
 };
 
+use super::jsonb::JsonbToJsonError;
+
 /// Value of a binlog event.
 #[derive(Debug, Clone, PartialEq)]
 pub enum BinlogValue<'a> {
@@ -67,7 +69,6 @@ impl<'de> MyDeserialize<'de> for BinlogValue<'de> {
                 if (byte0 & 0x30) != 0x30 {
                     // a long CHAR() field: see #37426
                     length = byte1 | (((byte0 & 0x30) ^ 0x30) << 4);
-                    col_type = ColumnType::try_from(byte0 as u8 | 0x30).unwrap_or(col_type);
                 } else {
                     length = byte1;
                 }
@@ -136,7 +137,7 @@ impl<'de> MyDeserialize<'de> for BinlogValue<'de> {
                 )))
             }
             MYSQL_TYPE_BIT => {
-                let nbits = col_meta[0] as usize * 8 + (col_meta[1] as usize);
+                let nbits = col_meta[1] as usize * 8 + (col_meta[0] as usize);
                 let nbytes = (nbits + 7) / 8;
                 let bytes: &[u8] = buf.parse(nbytes)?;
                 Ok(BinlogValue::Value(Bytes(bytes.into())))
@@ -244,3 +245,26 @@ impl<'de> MyDeserialize<'de> for BinlogValue<'de> {
         }
     }
 }
+
+#[derive(Debug, thiserror::Error)]
+pub enum BinlogValueToValueError {
+    #[error("Can't convert Jsonb to Json: {}", _0)]
+    ToJson(#[from] JsonbToJsonError),
+    #[error("Impossible to convert JsonDiff to Value")]
+    JsonDiff,
+}
+
+impl<'a> TryFrom<BinlogValue<'a>> for Value {
+    type Error = BinlogValueToValueError;
+
+    fn try_from(value: BinlogValue<'a>) -> Result<Self, Self::Error> {
+        match value {
+            BinlogValue::Value(x) => Ok(x),
+            BinlogValue::Jsonb(x) => {
+                let json = serde_json::Value::try_from(x)?;
+                Ok(Value::Bytes(Vec::from(json.to_string())))
+            }
+            BinlogValue::JsonDiff(_) => Err(BinlogValueToValueError::JsonDiff),
+        }
+    }
+}
