@@ -17,11 +17,12 @@ use super::{FromRow, FromRowError};
 use crate::{
     row::{new_row_raw, Row},
     value::{
-        convert::{ConvIr, FromValue, FromValueError},
+        convert::{FromValue, FromValueError},
         Value,
     },
 };
 
+#[cfg_attr(docsrs, doc(cfg(feature = "frunk")))]
 impl FromRow for HNil {
     fn from_row_opt(row: Row) -> Result<Self, FromRowError>
     where
@@ -34,9 +35,11 @@ impl FromRow for HNil {
     }
 }
 
+#[cfg_attr(docsrs, doc(cfg(feature = "frunk")))]
 impl<H, T> FromRow for HCons<H, T>
 where
     H: FromValue,
+    H::Intermediate: Into<Value>,
     T: FromRow + sealed::HlistFromRow,
     T: HList,
 {
@@ -76,6 +79,7 @@ mod sealed {
     impl<H, T> HlistFromRow for HCons<H, T>
     where
         H: FromValue,
+        H::Intermediate: Into<Value>,
         T: HlistFromRow,
         T: HList,
     {
@@ -102,12 +106,12 @@ mod sealed {
             let tail = match T::hlist_from_row_opt(values) {
                 Ok(t) => t,
                 Err(mut values) => {
-                    values.insert(0, Some(ir.rollback()));
+                    values.insert(0, Some(Into::<Value>::into(ir)));
                     return Err(values);
                 }
             };
 
-            Ok(h_cons(ir.commit(), tail))
+            Ok(h_cons(Into::<H>::into(ir), tail))
         }
     }
 }
